@@ -20,6 +20,7 @@
 #include <isc/buffer.h>
 #include <isc/log.h>
 #include <isc/mem.h>
+#include <isc/netmgr.h>
 #include <isc/parseint.h>
 #include <isc/region.h>
 #include <isc/result.h>
@@ -28,6 +29,7 @@
 #include <isc/util.h>
 
 #include <dns/fixedname.h>
+#include <dns/kasp.h>
 #include <dns/name.h>
 #include <dns/rdataclass.h>
 #include <dns/rdatatype.h>
@@ -59,11 +61,11 @@ options {\n\
 	edns-udp-size 1232;\n\
 	files unlimited;\n"
 #if defined(HAVE_GEOIP2)
-			    "	geoip-directory \"" MAXMINDDB_PREFIX "/share/"
-			    "GeoIP\";"
-			    "\n"
+			    "\
+	geoip-directory \"" MAXMINDDB_PREFIX "/share/GeoIP\";\n"
 #elif defined(HAVE_GEOIP2)
-			    "	geoip-directory \".\";\n"
+			    "\
+	geoip-directory \".\";\n"
 #endif /* if defined(HAVE_GEOIP2) */
 			    "\
 	heartbeat-interval 60;\n\
@@ -82,13 +84,22 @@ options {\n\
 	nta-lifetime 3600;\n\
 	nta-recheck 300;\n\
 #	pid-file \"" NAMED_LOCALSTATEDIR "/run/named/named.pid\"; \n\
-	port 53;\n\
+	port 53;\n"
+#if HAVE_SO_REUSEPORT_LB
+			    "\
+	reuseport yes;\n"
+#else
+			    "\
+	reuseport no;\n"
+#endif
+			    "\
 	tls-port 853;\n"
 #if HAVE_LIBNGHTTP2
-			    "http-port 80;\n"
-			    "https-port 443;\n"
-			    "http-listener-clients 300;\n"
-			    "http-streams-per-connection 100;\n"
+			    "\
+	http-port 80;\n\
+	https-port 443;\n\
+	http-listener-clients 300;\n\
+	http-streams-per-connection 100;\n"
 #endif
 			    "\
 	prefetch 2 9;\n\
@@ -126,6 +137,7 @@ options {\n\
 	trust-anchor-telemetry yes;\n\
 	udp-receive-buffer 0;\n\
 	udp-send-buffer 0;\n\
+	update-quota 100;\n\
 \n\
 	/* view */\n\
 	allow-new-zones no;\n\
@@ -284,6 +296,32 @@ view \"_bind\" chaos {\n\
 		database \"_builtin id\";\n\
 	};\n\
 };\n\
+"
+			    "#\n\
+#  Built-in DNSSEC key and signing policies.\n\
+#\n\
+dnssec-policy \"default\" {\n\
+	keys {\n\
+		csk key-directory lifetime unlimited algorithm 13;\n\
+	};\n\
+\n\
+	dnskey-ttl " DNS_KASP_KEY_TTL ";\n\
+	publish-safety " DNS_KASP_PUBLISH_SAFETY "; \n\
+	retire-safety " DNS_KASP_RETIRE_SAFETY "; \n\
+	purge-keys " DNS_KASP_PURGE_KEYS "; \n\
+	signatures-refresh " DNS_KASP_SIG_REFRESH "; \n\
+	signatures-validity " DNS_KASP_SIG_VALIDITY "; \n\
+	signatures-validity-dnskey " DNS_KASP_SIG_VALIDITY_DNSKEY "; \n\
+	max-zone-ttl " DNS_KASP_ZONE_MAXTTL "; \n\
+	zone-propagation-delay " DNS_KASP_ZONE_PROPDELAY "; \n\
+	parent-ds-ttl " DNS_KASP_DS_TTL "; \n\
+	parent-propagation-delay " DNS_KASP_PARENT_PROPDELAY "; \n\
+};\n\
+\n\
+dnssec-policy \"insecure\" {\n\
+	keys { };\n\
+};\n\
+\n\
 "
 			    "#\n\
 #  Default trusted key(s), used if \n\
@@ -325,6 +363,11 @@ named_config_parsedefaults(cfg_parser_t *parser, cfg_obj_t **conf) {
 				 CFG_PCTX_NODEPRECATED, conf));
 }
 
+const char *
+named_config_getdefault(void) {
+	return (defaultconf);
+}
+
 isc_result_t
 named_config_get(cfg_obj_t const *const *maps, const char *name,
 		 const cfg_obj_t **obj) {
@@ -354,7 +397,8 @@ named_checknames_get(const cfg_obj_t **maps, const char *const names[],
 	for (i = 0; maps[i] != NULL; i++) {
 		checknames = NULL;
 		if (cfg_map_get(maps[i], "check-names", &checknames) ==
-		    ISC_R_SUCCESS) {
+		    ISC_R_SUCCESS)
+		{
 			/*
 			 * Zone map entry is not a list.
 			 */
@@ -370,7 +414,8 @@ named_checknames_get(const cfg_obj_t **maps, const char *const names[],
 
 				for (size_t j = 0; names[j] != NULL; j++) {
 					if (strcasecmp(cfg_obj_asstring(type),
-						       names[j]) == 0) {
+						       names[j]) == 0)
+					{
 						*obj = cfg_tuple_get(value,
 								     "mode");
 						return (ISC_R_SUCCESS);
@@ -443,7 +488,8 @@ named_config_getzonetype(const cfg_obj_t *zonetypeobj) {
 	if (strcasecmp(str, "primary") == 0 || strcasecmp(str, "master") == 0) {
 		ztype = dns_zone_primary;
 	} else if (strcasecmp(str, "secondary") == 0 ||
-		   strcasecmp(str, "slave") == 0) {
+		   strcasecmp(str, "slave") == 0)
+	{
 		ztype = dns_zone_secondary;
 	} else if (strcasecmp(str, "mirror") == 0) {
 		ztype = dns_zone_mirror;
@@ -454,8 +500,7 @@ named_config_getzonetype(const cfg_obj_t *zonetypeobj) {
 	} else if (strcasecmp(str, "redirect") == 0) {
 		ztype = dns_zone_redirect;
 	} else {
-		INSIST(0);
-		ISC_UNREACHABLE();
+		UNREACHABLE();
 	}
 	return (ztype);
 }
@@ -463,19 +508,16 @@ named_config_getzonetype(const cfg_obj_t *zonetypeobj) {
 isc_result_t
 named_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 		       in_port_t defport, isc_mem_t *mctx,
-		       isc_sockaddr_t **addrsp, isc_dscp_t **dscpsp,
-		       uint32_t *countp) {
+		       isc_sockaddr_t **addrsp, uint32_t *countp) {
 	int count, i = 0;
-	const cfg_obj_t *addrlist;
-	const cfg_obj_t *portobj, *dscpobj;
-	const cfg_listelt_t *element;
-	isc_sockaddr_t *addrs;
+	const cfg_obj_t *addrlist = NULL;
+	const cfg_obj_t *portobj = NULL;
+	const cfg_listelt_t *element = NULL;
+	isc_sockaddr_t *addrs = NULL;
 	in_port_t port;
-	isc_dscp_t dscp = -1, *dscps = NULL;
 	isc_result_t result;
 
 	INSIST(addrsp != NULL && *addrsp == NULL);
-	INSIST(dscpsp == NULL || *dscpsp == NULL);
 	INSIST(countp != NULL);
 
 	addrlist = cfg_tuple_get(list, "addresses");
@@ -499,22 +541,6 @@ named_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 		}
 	}
 
-	if (dscpsp != NULL) {
-		dscpobj = cfg_tuple_get(list, "dscp");
-		if (dscpobj != NULL && cfg_obj_isuint32(dscpobj)) {
-			if (cfg_obj_asuint32(dscpobj) > 63) {
-				cfg_obj_log(dscpobj, named_g_lctx,
-					    ISC_LOG_ERROR,
-					    "dscp value '%u' is out of range",
-					    cfg_obj_asuint32(dscpobj));
-				return (ISC_R_RANGE);
-			}
-			dscp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
-		}
-
-		dscps = isc_mem_get(mctx, count * sizeof(isc_dscp_t));
-	}
-
 	addrs = isc_mem_get(mctx, count * sizeof(isc_sockaddr_t));
 
 	for (element = cfg_list_first(addrlist); element != NULL;
@@ -524,14 +550,6 @@ named_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 		INSIST(i < count);
 		addr = cfg_listelt_value(element);
 		addrs[i] = *cfg_obj_assockaddr(addr);
-		if (dscpsp != NULL) {
-			isc_dscp_t innerdscp;
-			innerdscp = cfg_obj_getdscp(addr);
-			if (innerdscp == -1) {
-				innerdscp = dscp;
-			}
-			dscps[i] = innerdscp;
-		}
 		if (isc_sockaddr_getport(&addrs[i]) == 0) {
 			isc_sockaddr_setport(&addrs[i], port);
 		}
@@ -541,26 +559,16 @@ named_config_getiplist(const cfg_obj_t *config, const cfg_obj_t *list,
 	*addrsp = addrs;
 	*countp = count;
 
-	if (dscpsp != NULL) {
-		*dscpsp = dscps;
-	}
-
 	return (ISC_R_SUCCESS);
 }
 
 void
 named_config_putiplist(isc_mem_t *mctx, isc_sockaddr_t **addrsp,
-		       isc_dscp_t **dscpsp, uint32_t count) {
+		       uint32_t count) {
 	INSIST(addrsp != NULL && *addrsp != NULL);
-	INSIST(dscpsp == NULL || *dscpsp != NULL);
 
 	isc_mem_put(mctx, *addrsp, count * sizeof(isc_sockaddr_t));
 	*addrsp = NULL;
-
-	if (dscpsp != NULL) {
-		isc_mem_put(mctx, *dscpsp, count * sizeof(isc_dscp_t));
-		*dscpsp = NULL;
-	}
 }
 
 static isc_result_t
@@ -582,7 +590,8 @@ getremotesdef(const cfg_obj_t *cctx, const char *list, const char *name,
 	while (elt != NULL) {
 		obj = cfg_listelt_value(elt);
 		if (strcasecmp(cfg_obj_asstring(cfg_tuple_get(obj, "name")),
-			       name) == 0) {
+			       name) == 0)
+		{
 			*ret = obj;
 			return (ISC_R_SUCCESS);
 		}
@@ -677,30 +686,28 @@ isc_result_t
 named_config_getipandkeylist(const cfg_obj_t *config, const char *listtype,
 			     const cfg_obj_t *list, isc_mem_t *mctx,
 			     dns_ipkeylist_t *ipkl) {
-	uint32_t addrcount = 0, dscpcount = 0, keycount = 0, tlscount = 0,
-		 i = 0;
+	uint32_t addrcount = 0, keycount = 0, tlscount = 0, i = 0;
 	uint32_t listcount = 0, l = 0, j;
 	uint32_t stackcount = 0, pushed = 0;
 	isc_result_t result;
 	const cfg_listelt_t *element;
 	const cfg_obj_t *addrlist;
 	const cfg_obj_t *portobj;
-	const cfg_obj_t *dscpobj;
 	in_port_t port = (in_port_t)0;
 	in_port_t def_port;
 	in_port_t def_tlsport;
-	isc_dscp_t dscp = -1;
 	isc_sockaddr_t *addrs = NULL;
-	isc_dscp_t *dscps = NULL;
 	dns_name_t **keys = NULL;
 	dns_name_t **tlss = NULL;
 	struct {
 		const char *name;
+		in_port_t port;
+		isc_sockaddr_t *src4s;
+		isc_sockaddr_t *src6s;
 	} *lists = NULL;
 	struct {
 		const cfg_listelt_t *element;
 		in_port_t port;
-		isc_dscp_t dscp;
 	} *stack = NULL;
 
 	REQUIRE(ipkl != NULL);
@@ -708,7 +715,6 @@ named_config_getipandkeylist(const cfg_obj_t *config, const char *listtype,
 	REQUIRE(ipkl->addrs == NULL);
 	REQUIRE(ipkl->keys == NULL);
 	REQUIRE(ipkl->tlss == NULL);
-	REQUIRE(ipkl->dscps == NULL);
 	REQUIRE(ipkl->labels == NULL);
 	REQUIRE(ipkl->allocated == 0);
 
@@ -725,15 +731,9 @@ named_config_getipandkeylist(const cfg_obj_t *config, const char *listtype,
 		goto cleanup;
 	}
 
-	result = named_config_getdscp(config, &dscp);
-	if (result != ISC_R_SUCCESS) {
-		goto cleanup;
-	}
-
 newlist:
 	addrlist = cfg_tuple_get(list, "addresses");
 	portobj = cfg_tuple_get(list, "port");
-	dscpobj = cfg_tuple_get(list, "dscp");
 
 	if (cfg_obj_isuint32(portobj)) {
 		uint32_t val = cfg_obj_asuint32(portobj);
@@ -746,17 +746,6 @@ newlist:
 		port = (in_port_t)val;
 	}
 
-	if (dscpobj != NULL && cfg_obj_isuint32(dscpobj)) {
-		if (cfg_obj_asuint32(dscpobj) > 63) {
-			cfg_obj_log(dscpobj, named_g_lctx, ISC_LOG_ERROR,
-				    "dscp value '%u' is out of range",
-				    cfg_obj_asuint32(dscpobj));
-			result = ISC_R_RANGE;
-			goto cleanup;
-		}
-		dscp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
-	}
-
 	result = ISC_R_NOMEMORY;
 
 	element = cfg_list_first(addrlist);
@@ -810,21 +799,15 @@ resume:
 			 */
 			stack[pushed].element = cfg_list_next(element);
 			stack[pushed].port = port;
-			stack[pushed].dscp = dscp;
 			pushed++;
 			goto newlist;
 		}
 
 		grow_array(mctx, addrs, i, addrcount);
-		grow_array(mctx, dscps, i, dscpcount);
 		grow_array(mctx, keys, i, keycount);
 		grow_array(mctx, tlss, i, tlscount);
 
 		addrs[i] = *cfg_obj_assockaddr(addr);
-		dscps[i] = cfg_obj_getdscp(addr);
-		if (dscps[i] == -1) {
-			dscps[i] = dscp;
-		}
 
 		result = named_config_getname(mctx, key, &keys[i]);
 		if (result != ISC_R_SUCCESS) {
@@ -840,30 +823,32 @@ resume:
 			goto cleanup;
 		}
 
-		/* Set the default port or tls-port */
-		if (port == 0) {
-			if (tlss[i] != NULL) {
-				port = def_tlsport;
-			} else {
-				port = def_port;
+		/* If the port is unset, take it from one of the upper levels */
+		if (isc_sockaddr_getport(&addrs[i]) == 0) {
+			in_port_t addr_port = port;
+
+			/* If unset, use the default port or tls-port */
+			if (addr_port == 0) {
+				if (tlss[i] != NULL) {
+					addr_port = def_tlsport;
+				} else {
+					addr_port = def_port;
+				}
 			}
+
+			isc_sockaddr_setport(&addrs[i], addr_port);
 		}
 
-		if (isc_sockaddr_getport(&addrs[i]) == 0) {
-			isc_sockaddr_setport(&addrs[i], port);
-		}
 		i++;
 	}
 	if (pushed != 0) {
 		pushed--;
 		element = stack[pushed].element;
 		port = stack[pushed].port;
-		dscp = stack[pushed].dscp;
 		goto resume;
 	}
 
 	shrink_array(mctx, addrs, i, addrcount);
-	shrink_array(mctx, dscps, i, dscpcount);
 	shrink_array(mctx, keys, i, keycount);
 	shrink_array(mctx, tlss, i, tlscount);
 
@@ -874,13 +859,10 @@ resume:
 		isc_mem_put(mctx, stack, stackcount * sizeof(stack[0]));
 	}
 
-	INSIST(dscpcount == addrcount);
 	INSIST(keycount == addrcount);
 	INSIST(tlscount == addrcount);
-	INSIST(keycount == dscpcount);
 
 	ipkl->addrs = addrs;
-	ipkl->dscps = dscps;
 	ipkl->keys = keys;
 	ipkl->tlss = tlss;
 	ipkl->count = addrcount;
@@ -892,9 +874,6 @@ cleanup:
 	if (addrs != NULL) {
 		isc_mem_put(mctx, addrs, addrcount * sizeof(addrs[0]));
 	}
-	if (dscps != NULL) {
-		isc_mem_put(mctx, dscps, dscpcount * sizeof(dscps[0]));
-	}
 	if (keys != NULL) {
 		for (j = 0; j < i; j++) {
 			if (keys[j] == NULL) {
@@ -957,32 +936,6 @@ named_config_getport(const cfg_obj_t *config, const char *type,
 	return (ISC_R_SUCCESS);
 }
 
-isc_result_t
-named_config_getdscp(const cfg_obj_t *config, isc_dscp_t *dscpp) {
-	const cfg_obj_t *options = NULL;
-	const cfg_obj_t *dscpobj = NULL;
-	isc_result_t result;
-
-	(void)cfg_map_get(config, "options", &options);
-	if (options == NULL) {
-		return (ISC_R_SUCCESS);
-	}
-
-	result = cfg_map_get(options, "dscp", &dscpobj);
-	if (result != ISC_R_SUCCESS || dscpobj == NULL) {
-		*dscpp = -1;
-		return (ISC_R_SUCCESS);
-	}
-	if (cfg_obj_asuint32(dscpobj) >= 64) {
-		cfg_obj_log(dscpobj, named_g_lctx, ISC_LOG_ERROR,
-			    "dscp '%u' out of range",
-			    cfg_obj_asuint32(dscpobj));
-		return (ISC_R_RANGE);
-	}
-	*dscpp = (isc_dscp_t)cfg_obj_asuint32(dscpobj);
-	return (ISC_R_SUCCESS);
-}
-
 struct keyalgorithms {
 	const char *str;
 	enum {
@@ -1067,8 +1020,7 @@ named_config_getkeyalgorithm2(const char *str, const dns_name_t **name,
 			*name = dns_tsig_hmacsha512_name;
 			break;
 		default:
-			INSIST(0);
-			ISC_UNREACHABLE();
+			UNREACHABLE();
 		}
 	}
 	if (typep != NULL) {
