@@ -11,10 +11,9 @@
 # See the COPYRIGHT file distributed with this work for additional
 # information regarding copyright ownership.
 
-SYSTEMTESTTOP=..
-. $SYSTEMTESTTOP/conf.sh
+. ../conf.sh
 
-$SHELL clean.sh
+$SHELL clean.sh 
 
 copy_setports ns2/named1.conf.in ns2/named.conf
 
@@ -27,9 +26,4 @@ copy_setports ns2/named-alt6.conf.in ns2/named-alt6.conf
 copy_setports ns2/named-alt7.conf.in ns2/named-alt7.conf
 
 mkdir ns2/nope
-
-if [ 1 = "${CYGWIN:-0}" ]; then
-  setfacl -s user::r-x,group::r-x,other::r-x ns2/nope
-else
-  chmod 555 ns2/nope
-fi
+chmod 555 ns2/nope
