@@ -172,7 +172,7 @@ disassociate_rdatasets(dns_validator_t *val) {
  * If we are validating a name that is marked as "must be secure", log a
  * warning and return DNS_R_MUSTBESECURE instead.
  */
-static inline isc_result_t
+static isc_result_t
 markanswer(dns_validator_t *val, const char *where, const char *mbstext) {
 	if (val->mustbesecure && mbstext != NULL) {
 		validator_log(val, ISC_LOG_WARNING,
@@ -195,7 +195,7 @@ markanswer(dns_validator_t *val, const char *where, const char *mbstext) {
 /*%
  * Mark the RRsets in val->event with trust level secure.
  */
-static inline void
+static void
 marksecure(dns_validatorevent_t *event) {
 	dns_rdataset_settrust(event->rdataset, dns_trust_secure);
 	if (event->sigrdataset != NULL) {
@@ -232,7 +232,7 @@ validator_done(dns_validator_t *val, isc_result_t result) {
 /*
  * Called when deciding whether to destroy validator 'val'.
  */
-static inline bool
+static bool
 exit_check(dns_validator_t *val) {
 	/*
 	 * Caller must be holding the lock.
@@ -425,7 +425,8 @@ fetch_callback_dnskey(isc_task_t *task, isc_event_t *event) {
 		 * Only extract the dst key if the keyset exists and is secure.
 		 */
 		if (eresult == ISC_R_SUCCESS &&
-		    rdataset->trust >= dns_trust_secure) {
+		    rdataset->trust >= dns_trust_secure)
+		{
 			result = select_signing_key(val, rdataset);
 			if (result == ISC_R_SUCCESS) {
 				val->keyset = &val->frdataset;
@@ -433,7 +434,8 @@ fetch_callback_dnskey(isc_task_t *task, isc_event_t *event) {
 		}
 		result = validate_answer(val, true);
 		if (result == DNS_R_NOVALIDSIG &&
-		    (val->attributes & VALATTR_TRIEDVERIFY) == 0) {
+		    (val->attributes & VALATTR_TRIEDVERIFY) == 0)
+		{
 			saved_result = result;
 			validator_log(val, ISC_LOG_DEBUG(3),
 				      "falling back to insecurity proof");
@@ -529,7 +531,7 @@ fetch_callback_ds(isc_task_t *task, isc_event_t *event) {
 			goto unexpected;
 		}
 
-	/* FALLTHROUGH */
+		FALLTHROUGH;
 	case ISC_R_SUCCESS:
 		if (trustchain) {
 			/*
@@ -668,7 +670,8 @@ validator_callback_dnskey(isc_task_t *task, isc_event_t *event) {
 		}
 		result = validate_answer(val, true);
 		if (result == DNS_R_NOVALIDSIG &&
-		    (val->attributes & VALATTR_TRIEDVERIFY) == 0) {
+		    (val->attributes & VALATTR_TRIEDVERIFY) == 0)
+		{
 			saved_result = result;
 			validator_log(val, ISC_LOG_DEBUG(3),
 				      "falling back to insecurity proof");
@@ -906,7 +909,8 @@ validator_callback_nsec(isc_task_t *task, isc_event_t *event) {
 				 * the wildcard used to generate the response.
 				 */
 				if (clabels == 0 ||
-				    dns_name_countlabels(wild) == clabels + 1) {
+				    dns_name_countlabels(wild) == clabels + 1)
+				{
 					val->attributes |= VALATTR_FOUNDCLOSEST;
 				}
 				/*
@@ -950,7 +954,7 @@ validator_callback_nsec(isc_task_t *task, isc_event_t *event) {
  * \li	DNS_R_NXDOMAIN
  * \li	DNS_R_BROKENCHAIN
  */
-static inline isc_result_t
+static isc_result_t
 view_find(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type) {
 	dns_fixedname_t fixedname;
 	dns_name_t *foundname;
@@ -1000,7 +1004,7 @@ notfound:
  * Checks to make sure we are not going to loop.  As we use a SHARED fetch
  * the validation process will stall if looping was to occur.
  */
-static inline bool
+static bool
 check_deadlock(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 	       dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset) {
 	dns_validator_t *parent;
@@ -1031,7 +1035,7 @@ check_deadlock(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 /*%
  * Start a fetch for the requested name and type.
  */
-static inline isc_result_t
+static isc_result_t
 create_fetch(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 	     isc_taskaction_t callback, const char *caller) {
 	unsigned int fopts = 0;
@@ -1062,7 +1066,7 @@ create_fetch(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 /*%
  * Start a subvalidation process.
  */
-static inline isc_result_t
+static isc_result_t
 create_validator(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 		 dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset,
 		 isc_taskaction_t action, const char *caller) {
@@ -1190,7 +1194,8 @@ seek_dnskey(dns_validator_t *val) {
 	namereln = dns_name_fullcompare(val->event->name, &siginfo->signer,
 					&order, &nlabels);
 	if (namereln != dns_namereln_subdomain &&
-	    namereln != dns_namereln_equal) {
+	    namereln != dns_namereln_equal)
+	{
 		return (DNS_R_CONTINUE);
 	}
 
@@ -1516,7 +1521,8 @@ validate_answer(dns_validator_t *val, bool resume) {
 	}
 
 	for (; result == ISC_R_SUCCESS;
-	     result = dns_rdataset_next(event->sigrdataset)) {
+	     result = dns_rdataset_next(event->sigrdataset))
+	{
 		dns_rdata_reset(&rdata);
 		dns_rdataset_current(event->sigrdataset, &rdata);
 		if (val->siginfo == NULL) {
@@ -2076,7 +2082,8 @@ checkwildcard(dns_validator_t *val, dns_rdatatype_t type,
 	     result = val_rdataset_next(val, &name, &rdataset))
 	{
 		if (rdataset->type != type ||
-		    rdataset->trust != dns_trust_secure) {
+		    rdataset->trust != dns_trust_secure)
+		{
 			continue;
 		}
 
@@ -2253,16 +2260,19 @@ findnsec3proofs(dns_validator_t *val) {
 			 * which proof.  Just populate them.
 			 */
 			if (NEEDNOQNAME(val) &&
-			    proofs[DNS_VALIDATOR_NOQNAMEPROOF] == NULL) {
+			    proofs[DNS_VALIDATOR_NOQNAMEPROOF] == NULL)
+			{
 				proofs[DNS_VALIDATOR_NOQNAMEPROOF] = name;
 			} else if (setclosest) {
 				proofs[DNS_VALIDATOR_CLOSESTENCLOSER] = name;
 			} else if (NEEDNODATA(val) &&
-				   proofs[DNS_VALIDATOR_NODATAPROOF] == NULL) {
+				   proofs[DNS_VALIDATOR_NODATAPROOF] == NULL)
+			{
 				proofs[DNS_VALIDATOR_NODATAPROOF] = name;
 			} else if (NEEDNOWILDCARD(val) &&
 				   proofs[DNS_VALIDATOR_NOWILDCARDPROOF] ==
-					   NULL) {
+					   NULL)
+			{
 				proofs[DNS_VALIDATOR_NOWILDCARDPROOF] = name;
 			}
 			return (result);
@@ -2407,7 +2417,8 @@ validate_authority(dns_validator_t *val, bool resume) {
 		}
 
 		for (; rdataset != NULL;
-		     rdataset = ISC_LIST_NEXT(rdataset, link)) {
+		     rdataset = ISC_LIST_NEXT(rdataset, link))
+		{
 			if (rdataset->type == dns_rdatatype_rrsig) {
 				continue;
 			}
@@ -3025,7 +3036,8 @@ validator_start(isc_task_t *task, isc_event_t *event) {
 			result = validate_answer(val, false);
 		}
 		if (result == DNS_R_NOVALIDSIG &&
-		    (val->attributes & VALATTR_TRIEDVERIFY) == 0) {
+		    (val->attributes & VALATTR_TRIEDVERIFY) == 0)
+		{
 			saved_result = result;
 			validator_log(val, ISC_LOG_DEBUG(3),
 				      "falling back to insecurity proof");
@@ -3035,7 +3047,8 @@ validator_start(isc_task_t *task, isc_event_t *event) {
 			}
 		}
 	} else if (val->event->rdataset != NULL &&
-		   val->event->rdataset->type != 0) {
+		   val->event->rdataset->type != 0)
+	{
 		/*
 		 * This is either an unsecure subdomain or a response
 		 * from a broken server.
@@ -3051,7 +3064,8 @@ validator_start(isc_task_t *task, isc_event_t *event) {
 				      "parent indicates it should be secure");
 		}
 	} else if ((val->event->rdataset == NULL &&
-		    val->event->sigrdataset == NULL)) {
+		    val->event->sigrdataset == NULL))
+	{
 		/*
 		 * This is a validation of a negative response.
 		 */
@@ -3068,7 +3082,8 @@ validator_start(isc_task_t *task, isc_event_t *event) {
 
 		result = validate_nx(val, false);
 	} else if ((val->event->rdataset != NULL &&
-		    NEGATIVE(val->event->rdataset))) {
+		    NEGATIVE(val->event->rdataset)))
+	{
 		/*
 		 * This is a delayed validation of a negative cache entry.
 		 */
@@ -3085,8 +3100,7 @@ validator_start(isc_task_t *task, isc_event_t *event) {
 
 		result = validate_nx(val, false);
 	} else {
-		INSIST(0);
-		ISC_UNREACHABLE();
+		UNREACHABLE();
 	}
 
 	if (result != DNS_R_WAIT) {
